@@ -1,3 +1,6 @@
+/* Workaround perl5 global namespace pollution. Note that undefining library
+ * functions like fopen will not solve the problem on all platforms as fopen
+ * might be a macro on Windows but not necessarily on other operating systems. */
 #ifdef do_open
   #undef do_open
 #endif
@@ -49,9 +52,6 @@
 #ifdef ref
   #undef ref
 #endif
-#ifdef ENTER
-  #undef ENTER
-#endif
 #ifdef read
   #undef read
 #endif
@@ -70,4 +70,18 @@
 #ifdef rewind
   #undef rewind
 #endif
-
+#ifdef free
+  #undef free
+#endif
+#ifdef malloc
+  #undef malloc
+#endif
+#ifdef calloc
+  #undef calloc
+#endif
+#ifdef Stat
+  #undef Stat
+#endif
+#ifdef check
+  #undef check
+#endif
