@@ -8,12 +8,19 @@
  * Copyright (C) 1999-2000.  The University of Chicago
  * See the file LICENSE for information on usage and redistribution.
  *
- * $Header: /cvsroot/swig/SWIG/Source/DOH/doh.h,v 1.10 2005/02/15 21:00:52 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/DOH/doh.h,v 1.21 2006/02/19 22:16:26 wsfulton Exp $
  * ----------------------------------------------------------------------------- */
 
 #ifndef _DOH_H
 #define _DOH_H
 
+#ifndef MACSWIG
+#include "swigconfig.h"
+#endif
+
+#include <stdio.h>
+#include <stdarg.h>
+
 /* Set the namespace prefix for DOH API functions. This can be used to control
    visibility of the functions in libraries */
 
@@ -40,8 +47,9 @@
 #define DohLen             DOH_NAMESPACE(Len)
 #define DohHashval         DOH_NAMESPACE(Hashval)
 #define DohCmp             DOH_NAMESPACE(Cmp)
+#define DohEqual           DOH_NAMESPACE(Equal)
 #define DohIncref          DOH_NAMESPACE(Incref)
-#define DohGetattr         DOH_NAMESPACE(Getattr)
+#define DohCheckattr       DOH_NAMESPACE(Checkattr)
 #define DohSetattr         DOH_NAMESPACE(Setattr)
 #define DohDelattr         DOH_NAMESPACE(Delattr)
 #define DohKeys            DOH_NAMESPACE(Keys)
@@ -65,6 +73,17 @@
 #define DohGetc            DOH_NAMESPACE(Getc)
 #define DohPutc            DOH_NAMESPACE(Putc)
 #define DohUngetc          DOH_NAMESPACE(Ungetc)
+
+#define DohStringPutc      DOH_NAMESPACE(StringPutc)
+#define DohStringGetc      DOH_NAMESPACE(StringGetc)
+#define DohStringUngetc    DOH_NAMESPACE(StringUngetc)
+#define DohStringAppend    DOH_NAMESPACE(StringAppend)
+#define DohStringLen       DOH_NAMESPACE(StringLen)
+#define DohStringChar      DOH_NAMESPACE(StringChar)
+#define DohStringEqual     DOH_NAMESPACE(StringEqual)
+
+
+
 #define DohGetline         DOH_NAMESPACE(Getline)
 #define DohSetline         DOH_NAMESPACE(Setline)
 #define DohGetfile         DOH_NAMESPACE(Getfile)
@@ -84,6 +103,7 @@
 #define DohIsString        DOH_NAMESPACE(IsString)
 #define DohIsFile          DOH_NAMESPACE(IsFile)
 #define DohNewString       DOH_NAMESPACE(NewString)
+#define DohNewStringEmpty  DOH_NAMESPACE(NewStringEmpty)
 #define DohNewStringWithSize  DOH_NAMESPACE(NewStringWithSize)
 #define DohNewStringf      DOH_NAMESPACE(NewStringf)
 #define DohStrcmp          DOH_NAMESPACE(Strcmp)
@@ -93,10 +113,13 @@
 #define DohNewFile         DOH_NAMESPACE(NewFile)
 #define DohNewFileFromFile DOH_NAMESPACE(NewFileFromFile)
 #define DohNewFileFromFd   DOH_NAMESPACE(NewFileFromFd)
+#define DohFileErrorDisplay   DOH_NAMESPACE(FileErrorDisplay)
 #define DohClose           DOH_NAMESPACE(Close)
 #define DohCopyto          DOH_NAMESPACE(Copyto)
 #define DohNewList         DOH_NAMESPACE(NewList)
 #define DohNewHash         DOH_NAMESPACE(NewHash)
+#define DohHashGetAttr     DOH_NAMESPACE(HashGetAttr)
+#define DohHashCheckAttr   DOH_NAMESPACE(HashCheckAttr)
 #define DohNewVoid         DOH_NAMESPACE(NewVoid)
 #define DohSplit           DOH_NAMESPACE(Split)
 #define DohSplitLines      DOH_NAMESPACE(SplitLines)
@@ -115,9 +138,6 @@
 #define DohNext            DOH_NAMESPACE(Next)
 #endif
 
-#include <stdio.h>
-#include <stdarg.h>
-
 #define DOH_MAJOR_VERSION 0
 #define DOH_MINOR_VERSION 1
 
@@ -178,6 +198,7 @@ extern int           DohDump(const DOH *obj, DOHFile *out);
 extern int           DohLen(const DOH *obj);
 extern int           DohHashval(const DOH *obj);
 extern int           DohCmp(const DOH *obj1, const DOH *obj2);
+extern int           DohEqual(const DOH *obj1, const DOH *obj2);
 extern void          DohIncref(DOH *obj);
 
 /* Mapping methods */
@@ -187,10 +208,15 @@ extern int           DohSetattr(DOH *obj, const DOHString_or_char *name, const D
 extern int           DohDelattr(DOH *obj, const DOHString_or_char *name);
 extern DOH          *DohKeys(DOH *obj);
 extern int           DohGetInt(DOH *obj, const DOHString_or_char *name);
-extern double        DohGetDouble(DOH *obj, const DOHString_or_char *name);
-extern char         *DohGetChar(DOH *obj, const DOHString_or_char *name);
 extern void          DohSetInt(DOH *obj, const DOHString_or_char *name, int);
+extern double        DohGetDouble(DOH *obj, const DOHString_or_char *name);
 extern void          DohSetDouble(DOH *obj, const DOHString_or_char *name, double);
+extern char         *DohGetChar(DOH *obj, const DOHString_or_char *name);
+extern void          DohSetChar(DOH *obj, const DOH *name, char *value);
+extern void         *DohGetFlagAttr(DOH *obj, const DOHString_or_char *name);
+extern int           DohGetFlag(DOH *obj, const DOHString_or_char *name);
+extern void          DohSetFlagAttr(DOH *obj, const DOHString_or_char *name, const DOHString_or_char *attr);
+extern void          DohSetFlag(DOH *obj, const DOHString_or_char *name);
 extern void         *DohGetVoid(DOH *obj, const DOHString_or_char *name);
 extern void          DohSetVoid(DOH *obj, const DOHString_or_char *name, void *value);
 
@@ -212,6 +238,8 @@ extern int           DohGetc(DOHFile *obj);
 extern int           DohPutc(int ch, DOHFile *obj);
 extern int           DohUngetc(int ch, DOHFile *obj);
 
+
+
 /* Iterators */
 extern DohIterator   DohFirst(DOH *obj);
 extern DohIterator   DohNext(DohIterator x);
@@ -228,6 +256,22 @@ extern void          DohSetfile(DOH *obj, DOH *file);
 extern int           DohReplace(DOHString *src, const DOHString_or_char *token, const DOHString_or_char *rep, int flags);
 extern void          DohChop(DOHString *src);
 
+extern int DohString_putc(DOH *so, int ch);
+extern int DohString_getc(DOH *so);
+extern int DohString_ungetc(DOH *so, int ch);
+extern void DohString_append(DOH *so, DOH *str);
+extern int DohString_len(DOH *s1);
+extern char *DohString_char(DOH *s1);
+extern int DohString_equal(DOH *s1, DOH *s2);
+
+#define DohStringPutc(ch,so)    DohString_putc(so, ch)
+#define DohStringGetc(so)       DohString_getc(so)
+#define DohStringUngetc(ch,so)  DohString_ungetc(so, ch)
+#define DohStringAppend(so,str) DohString_append(so, (DOH*)str)
+#define DohStringLen(so)        DohString_len((DOH*)so)
+#define DohStringChar(so)       DohString_char(so)
+#define DohStringEqual(s1,s2)   DohString_equal((DOH *)s1, (DOH *)s2)
+
 /* Meta-variables */
 extern DOH          *DohGetmeta(DOH *, const DOH *);
 extern int           DohSetmeta(DOH *, const DOH *, const DOH *value);
@@ -255,6 +299,7 @@ extern int           DohGetmark(DOH *obj);
  * Strings.
  * ----------------------------------------------------------------------------- */
 
+extern DOHString    *DohNewStringEmpty();
 extern DOHString    *DohNewString(const DOH *c);
 extern DOHString    *DohNewStringWithSize(const DOH *c, int len);
 extern DOHString    *DohNewStringf(const DOH *fmt, ...);
@@ -283,6 +328,7 @@ extern char *DohStrchr(const DOHString_or_char *s1, int ch);
 extern DOHFile *DohNewFile(DOH *file, const char *mode);
 extern DOHFile *DohNewFileFromFile(FILE *f);
 extern DOHFile *DohNewFileFromFd(int fd);
+extern void     DohFileErrorDisplay(DOHString *filename);
 extern int      DohClose(DOH *file);
 extern int      DohCopyto(DOHFile *input, DOHFile *output);
 
@@ -299,6 +345,8 @@ extern void DohSortList(DOH *lo, int (*cmp)(const DOH *, const DOH *));
  * ----------------------------------------------------------------------------- */
 
 extern DOHHash   *DohNewHash();
+extern DOH       *DohHashGetAttr(DOH *hash, const DOH *key);
+extern int       DohHashCheckAttr(DOH *hash, DOH *key, DOH *value);
 
 /* -----------------------------------------------------------------------------
  * Void
@@ -335,6 +383,7 @@ extern void      DohMemoryDebug(void);
 #define Data               DohData
 #define Char               (char *) Data
 #define Cmp                DohCmp
+#define Equal              DohEqual
 #define Setline            DohSetline
 #define Getline            DohGetline
 #define Setfile            DohSetfile
@@ -348,16 +397,31 @@ extern void      DohMemoryDebug(void);
 #define Getc               DohGetc
 #define Putc               DohPutc
 #define Ungetc             DohUngetc
+
+#define StringPutc         DohStringPutc
+#define StringGetc         DohStringGetc
+#define StringUngetc       DohStringUngetc
+#define StringAppend       DohStringAppend
+#define StringLen          DohStringLen
+#define StringChar         DohStringChar
+#define StringEqual        DohStringEqual
+
 #define Close              DohClose
 #define vPrintf            DohvPrintf
 #define GetInt             DohGetInt
 #define GetDouble          DohGetDouble
 #define GetChar            DohGetChar
 #define GetVoid            DohGetVoid
+#define GetFlagAttr        DohGetFlagAttr
+#define GetFlag            DohGetFlag
 #define SetInt             DohSetInt
 #define SetDouble          DohSetDouble
 #define SetChar            DohSetattr
 #define SetVoid            DohSetVoid
+#define SetFlagAttr        DohSetFlagAttr
+#define SetFlag            DohSetFlag
+#define UnsetFlag(o,n)     DohSetFlagAttr(o,n,NULL)
+#define ClearFlag(o,n)     DohSetFlagAttr(o,n,"")
 #define Readline           DohReadline
 #define Replace            DohReplace
 #define Chop               DohChop
@@ -365,13 +429,17 @@ extern void      DohMemoryDebug(void);
 #define Setmeta            DohSetmeta
 #define Delmeta            DohDelmeta
 #define NewString          DohNewString
+#define NewStringEmpty     DohNewStringEmpty
 #define NewStringWithSize  DohNewStringWithSize
 #define NewStringf         DohNewStringf
 #define NewHash            DohNewHash
+#define HashGetAttr        DohHashGetAttr
+#define HashCheckAttr      DohHashCheckAttr
 #define NewList            DohNewList
 #define NewFile            DohNewFile
 #define NewFileFromFile    DohNewFileFromFile
 #define NewFileFromFd      DohNewFileFromFd
+#define FileErrorDisplay   DohFileErrorDisplay
 #define Close              DohClose
 #define NewVoid            DohNewVoid
 #define Keys               DohKeys
