@@ -1,27 +1,13 @@
-/********************************************************************
- * CHICKEN module for SWIG
- *
- * Author : Jonah Beckford
- * 
- * Major restructuring: John Lenz
- *
- * Modified from mzscheme.cxx, guile.cxx and python.cxx : David
- * Beazley
- *
- * Please read the file LICENSE for the copyright and terms by which
- * SWIG can be used and distributed.
- *
- ********************************************************************/
-
-char cvsroot_chicken_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/chicken.cxx,v 1.42 2005/04/16 21:42:42 wuzzeb Exp $";
-
-/***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/chicken.cxx,v 1.42 2005/04/16 21:42:42 wuzzeb Exp $
+/* -----------------------------------------------------------------------------
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
  *
  * chicken.cxx
  *
- * Definitions for adding functions to CHICKEN
- ***********************************************************************/
+ * CHICKEN language module for SWIG.
+ * ----------------------------------------------------------------------------- */
+
+char cvsroot_chicken_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/chicken.cxx,v 1.52 2006/03/06 22:50:56 wsfulton Exp $";
 
 #include "swigmod.h"
 
@@ -63,6 +49,7 @@ static int           clos_uses = 1;
 
 /* C++ Support + Clos Classes */
 static int           clos = 0;
+static  String       *c_class_name = 0;
 static  String       *class_name  = 0;
 static  String       *short_class_name  = 0;
 
@@ -207,7 +194,7 @@ CHICKEN::top(Node *n)
     
   f_runtime = NewFile(outfile,"w");
   if (!f_runtime) {
-    Printf(stderr,"*** Can't open '%s'\n", outfile);
+    FileErrorDisplay(outfile);
     SWIG_exit(EXIT_FAILURE);
   }
   f_init = NewString("");
@@ -268,7 +255,7 @@ CHICKEN::top(Node *n)
 
   Printf(chicken_filename,"%s%s.scm", SWIG_output_directory(), module);
   if ((f_scm = NewFile(chicken_filename,"w")) == 0) {
-    Printf(stderr,"Unable to open %s\n", chicken_filename);
+    FileErrorDisplay(chicken_filename);
     SWIG_exit(EXIT_FAILURE);
   }
 
@@ -357,10 +344,10 @@ CHICKEN::functionWrapper(Node *n)
   String *scmname;
   bool any_specialized_arg = false;
   List *function_arg_types = NewList();
-  int return_proxy_fastcall = 0;
 
   int     num_required;
   int     num_arguments;
+  int     have_argout;
 
   Printf(mangle, "\"%s\"", SwigType_manglestr(d));
 
@@ -529,12 +516,16 @@ CHICKEN::functionWrapper(Node *n)
   emit_action(n,f);
 
   /* Insert argument output code */
+  have_argout = 0;
   for (p = l; p;) {
     if ((tm = Getattr(p,"tmap:argout"))) {
-      if (!Wrapper_check_local(f, "gswig_list_p")) {
-        Wrapper_add_local(f,"resultobjlast", "C_word resultobjlast");
-        Wrapper_add_local(f,"gswig_list_p", "int gswig_list_p = 0");
+      
+      if (!have_argout) {
+        have_argout = 1;
+        // Print initial argument output code
+        Printf(f->code,"SWIG_Chicken_SetupArgout\n");
       }
+      
       Replaceall(tm,"$source",Getattr(p,"lname"));
       Replaceall(tm,"$target","resultobj");
       Replaceall(tm,"$arg",Getattr(p,"emit:input"));
@@ -551,28 +542,17 @@ CHICKEN::functionWrapper(Node *n)
     Replaceall(tm,"$source", "result");
     Replaceall(tm,"$target", "resultobj");
     Replaceall(tm,"$result", "resultobj");
-    if (Getattr(n,"feature:new")) {
+    if (GetFlag(n,"feature:new")) {
       Replaceall(tm,"$owner","1");
     } else {
       Replaceall(tm,"$owner","0");
     }
 
-    /* check for chickenfastproxy flag */
-    if (Getattr(n, "tmap:out:chickenfastproxy")) {
-      if (exporting_constructor && clos && hide_primitive) {
-        /* Don't return a proxy, the wrapped CLOS class is the proxy */
-        Replaceall(tm, "$proxy", "0");
-      } else {
-        /* can only do fast proxy if there are no argout paramaters... */
-        if (Wrapper_check_local(f, "gswig_list_p")) {
-          Replaceall(tm, "$proxy", "1");
-        } else {
-          Replaceall(tm, "$proxy", "0");
-          return_proxy_fastcall = 1;
-        }
-      }
-    }
     Printf(f->code, "%s", tm);
+
+    if (have_argout)
+      Printf(f->code, "\nSWIG_APPEND_VALUE(resultobj);\n");
+      
   } else {
     Swig_warning(WARN_TYPEMAP_OUT_UNDEF, input_file, line_number,
                  "Unable to use return type %s in function %s.\n", 
@@ -586,7 +566,7 @@ CHICKEN::functionWrapper(Node *n)
   Printv(f->code,cleanup,NIL);
 
   /* Look to see if there is any newfree cleanup code */
-  if (Getattr(n,"feature:new")) {
+  if (GetFlag(n,"feature:new")) {
     if ((tm = Swig_typemap_lookup_new("newfree",n,"result",0))) {
       Replaceall(tm,"$source","result");
       Printf(f->code,"%s\n",tm);
@@ -599,22 +579,24 @@ CHICKEN::functionWrapper(Node *n)
     Printf(f->code,"%s\n",tm);
   }
 
-  if (return_proxy_fastcall) {
-    Printv(f->code,"{\n",
+
+  if (have_argout) {
+    Printf(f->code, "C_kontinue(continuation,C_SCHEME_END_OF_LIST);\n");
+  } else {
+      if (exporting_constructor && clos && hide_primitive) {
+        /* Don't return a proxy, the wrapped CLOS class is the proxy */
+        Printf(f->code, "C_kontinue(continuation,resultobj);\n");
+      } else {
+        // make the continuation the proxy creation function, if one exists
+        Printv(f->code,"{\n",
                    "C_word func;\n",
-                   "SWIG_Chicken_FindCreateProxy(func, resultobj);\n",
+                   "SWIG_Chicken_FindCreateProxy(func, resultobj)\n",
                    "if (C_swig_is_closurep(func))\n",
-                   "  ((C_proc3)(void *)C_block_item(func, 0))(3,func,continuation,resultobj);\n",
+                   "  ((C_proc4)(void *)C_block_item(func, 0))(4,func,continuation,resultobj,C_SCHEME_FALSE);\n",
                    "else\n",
                    "  C_kontinue(continuation, resultobj);\n",
                    "}\n", NIL);
-  } else if (Wrapper_check_local(f, "gswig_list_p")) {
-    Printv(f->code,"if (gswig_list_p)\n",
-		   "  C_apply_values(3, C_SCHEME_UNDEFINED, continuation, resultobj);\n",
-		   "else\n",
-		   "  C_kontinue(continuation, resultobj);\n", NIL);
-  } else {
-    Printf(f->code,"C_kontinue (continuation, resultobj);\n");
+      }
   }
 
   /* Error handling code */
@@ -771,13 +753,14 @@ CHICKEN::variableWrapper(Node *n)  {
     Printf(f->code, "if (argc!=2 && argc!=3) C_bad_argc(argc,2);\n");
 
     /* Check for a setting of the variable value */
-    if (!Getattr(n,"feature:immutable")) {
+    if (!GetFlag(n,"feature:immutable")) {
       Printf(f->code, "if (argc > 2) {\n");
       if ((tm = Swig_typemap_lookup_new("varin",n,name,0))) {
         Replaceall(tm,"$source","value");
         Replaceall(tm,"$target",name);
         Replaceall(tm,"$input","value");
-        Printv(f->code, tm, "\n",NIL);
+        /* Printv(f->code, tm, "\n",NIL); */
+	emit_action_code(n, f, tm);
       }
       else {
         Swig_warning(WARN_TYPEMAP_VARIN_UNDEF, input_file, line_number, 
@@ -801,25 +784,22 @@ CHICKEN::variableWrapper(Node *n)  {
       Replaceall(tm,"$varname",varname);
       Replaceall(tm,"$target","resultobj");
       Replaceall(tm,"$result","resultobj");
-      Printf(f->code, "%s\n", tm);
+      /* Printf(f->code, "%s\n", tm); */
+      emit_action_code(n, f, tm);
     }
     else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number,
                    "Unable to read variable of type %s\n", SwigType_str(t,0));
     }
 
-    if (Getattr(n, "tmap:varout:chickenfastproxy")) {
-      Printv(f->code,"{\n",
-                     "C_word func;\n",
-                     "SWIG_Chicken_FindCreateProxy(func, resultobj);\n",
-                     "if (C_swig_is_closurep(func))\n",
-                     "  ((C_proc3)(void *)C_block_item(func, 0))(3,func,continuation,resultobj);\n",
-                     "else\n",
-                     "  C_kontinue(continuation, resultobj);\n",
-                     "}\n", NIL);
-    } else {
-      Printf(f->code,"    C_kontinue (continuation, resultobj);\n");
-    }
+    Printv(f->code,"{\n",
+                   "C_word func;\n",
+                   "SWIG_Chicken_FindCreateProxy(func, resultobj)\n",
+                   "if (C_swig_is_closurep(func))\n",
+                   "  ((C_proc4)(void *)C_block_item(func, 0))(4,func,continuation,resultobj,C_SCHEME_FALSE);\n",
+                   "else\n",
+                   "  C_kontinue(continuation, resultobj);\n",
+                   "}\n", NIL);
 
     /* Error handling code */
 #ifdef USE_FAIL
@@ -842,9 +822,11 @@ CHICKEN::variableWrapper(Node *n)  {
                 clos_name = chickenNameMapping(scmname, (char *)"");
 
         Node *class_node = classLookup(t);
-        if (class_node && Getattr(n, "tmap:varout:chickenfastproxy")) {
+        String *clos_code = Getattr(n, "tmap:varin:closcode");
+        if (class_node && clos_code) {
+          Replaceall(clos_code,"$input", "(car lst)");
           Printv(clos_methods, "(define (", clos_name, " . lst) (if (null? lst) (", chickenPrimitiveName(scmname), ") (",
-                     chickenPrimitiveName(scmname), " (slot-ref (car lst) 'swig-this))))\n", NIL);
+                     chickenPrimitiveName(scmname), " ", clos_code, ")))\n", NIL);
         } else {
           /* Simply re-export the procedure */
           Printv(clos_methods, "(define ", clos_name, " ", chickenPrimitiveName(scmname), ")\n", NIL);
@@ -989,25 +971,22 @@ CHICKEN::constantWrapper(Node *n)
       Replaceall(tm,"$varname",source);
       Replaceall(tm,"$target","resultobj");
       Replaceall(tm,"$result","resultobj");
-      Printf(f->code, "%s\n", tm);
+      /* Printf(f->code, "%s\n", tm); */
+      emit_action_code(n, f, tm);
     }
     else {
       Swig_warning(WARN_TYPEMAP_VAROUT_UNDEF, input_file, line_number,
                    "Unable to read variable of type %s\n", SwigType_str(t,0));
     }
 
-    if (Getattr(n, "tmap:varout:chickenfastproxy")) {
-      Printv(f->code,"{\n",
-                     "C_word func;\n",
-                     "SWIG_Chicken_FindCreateProxy(func, resultobj);\n",
-                     "if (C_swig_is_closurep(func))\n",
-                     "  ((C_proc3)(void *)C_block_item(func, 0))(3,func,continuation,resultobj);\n",
-                     "else\n",
-                     "  C_kontinue(continuation, resultobj);\n",
-                     "}\n", NIL);
-    } else {
-      Printf(f->code,"    C_kontinue (continuation, resultobj);\n");
-    }
+    Printv(f->code,"{\n",
+                   "C_word func;\n",
+                   "SWIG_Chicken_FindCreateProxy(func, resultobj)\n",
+                   "if (C_swig_is_closurep(func))\n",
+                   "  ((C_proc4)(void *)C_block_item(func, 0))(4,func,continuation,resultobj,C_SCHEME_FALSE);\n",
+                   "else\n",
+                   "  C_kontinue(continuation, resultobj);\n",
+                   "}\n", NIL);
 
     /* Error handling code */
 #ifdef USE_FAIL
@@ -1059,29 +1038,28 @@ CHICKEN::classHandler(Node *n)
   constructor_dispatch = 0;
   constructor_name = 0;
       
+  c_class_name = NewString(Getattr(n,"sym:name"));
   class_name = NewString("");
   short_class_name = NewString("");
-  Printv(class_name, "<", Getattr(n,"sym:name"), ">", NIL);
-  Printv(short_class_name, Getattr(n,"sym:name"), NIL);
+  Printv(class_name, "<", c_class_name, ">", NIL);
+  Printv(short_class_name, c_class_name, NIL);
   Replaceall(class_name, "_", "-");
   Replaceall(short_class_name, "_", "-");
       
   if (!addSymbol(class_name,n)) return SWIG_ERROR;
 
   /* Handle inheritance */
-  String *base_class = NewString("<");
+  String *base_class = NewString("");
   List *baselist = Getattr(n,"bases");
   if (baselist && Len(baselist)) {
     Iterator base = First(baselist);
     while (base.item) {
-        Printv(base_class, Getattr(base.item, "sym:name"),NIL);
-        base = Next(base);
-        if (base.item) {
-          Printf(base_class, "> <");
-        }
-      }
+      if (!Getattr(base.item, "feature:ignore"))
+        Printv(base_class, "<", Getattr(base.item, "sym:name"), "> ", NIL);
+      base = Next(base);
     }
-    Printf(base_class, ">");
+  }
+
     Replaceall(base_class, "_", "-");
 
     String *scmmod = NewString(module);
@@ -1091,7 +1069,7 @@ CHICKEN::classHandler(Node *n)
                     "  (make <swig-metaclass-", scmmod, "> 'name \"", short_class_name, "\"\n", NIL);
     Delete(scmmod);
 
-    if (Len(base_class) > 2) {
+    if (Len(base_class)) {
       Printv(clos_class_defines,"    'direct-supers (list ", base_class, ")\n", NIL);
     } else {
       Printv(clos_class_defines,"    'direct-supers (list <object>)\n", NIL);
@@ -1163,8 +1141,9 @@ CHICKEN::classHandler(Node *n)
                          "}\n", NIL);
       addMethod(closfuncname, funcname);
 
-      Printv(clos_methods, "(", chickenPrimitiveName(closfuncname), " (lambda (x) (make ", class_name, " 'swig-this x)))\n\n", NIL);
-
+      Printv(clos_methods, "(", chickenPrimitiveName(closfuncname), " (lambda (x lst) (if lst ",
+                                                                                        "(cons (make ", class_name, " 'swig-this x) lst) ",
+                                                                                        "(make ", class_name, " 'swig-this x))))\n\n", NIL);
       Delete(closfuncname);
       Delete(funcname);
   }
@@ -1175,8 +1154,10 @@ CHICKEN::classHandler(Node *n)
                            
   Delete(class_name);
   Delete(short_class_name);
+  Delete(c_class_name);
   class_name = 0;
   short_class_name = 0;
+  c_class_name = 0;
 
   return SWIG_OK;
 }
@@ -1227,12 +1208,16 @@ CHICKEN::membervariableHandler(Node *n)
   //Node *class_node = Swig_symbol_clookup(pb, Getattr(n, "sym:symtab"));
   Node *class_node = classLookup(Getattr(n, "type"));
 
-  String *getfunc = NewStringf("%s-%s-get", short_class_name, proc);
-  String *setfunc = NewStringf("%s-%s-set", short_class_name, proc);
+  //String *getfunc = NewStringf("%s-%s-get", short_class_name, proc);
+  //String *setfunc = NewStringf("%s-%s-set", short_class_name, proc);
+  String *getfunc = Swig_name_get(Swig_name_member(c_class_name, iname));
+  Replaceall(getfunc, "_","-");
+  String *setfunc = Swig_name_set(Swig_name_member(c_class_name, iname));
+  Replaceall(setfunc, "_","-");
 
   Printv(clos_class_defines,"        (list '", proc, " ':swig-virtual ':swig-get ", chickenPrimitiveName(getfunc), NIL);
 
-  if (!Getattr(n,"feature:immutable")) {
+  if (!GetFlag(n,"feature:immutable")) {
       if (class_node) {
           Printv(clos_class_defines, " ':swig-set (lambda (x y) (", chickenPrimitiveName(setfunc), " x (slot-ref y 'swig-this))))\n", NIL);
       } else {
@@ -1282,9 +1267,19 @@ CHICKEN::constructorHandler(Node *n)
 }
 
 int CHICKEN::destructorHandler(Node *n) {
+
+  if (no_collection)
+    member_name = NewStringf("delete-%s", short_class_name);
+
   exporting_destructor = true;
   Language::destructorHandler(n);
   exporting_destructor = false;
+
+  if (no_collection) {
+    Delete(member_name);
+    member_name = NULL;
+  }
+
   return SWIG_OK;
 }
 
