@@ -1,19 +1,17 @@
 /* -----------------------------------------------------------------------------
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ *
  * csharp.cxx
  *
- *     CSharp wrapper module.
- *
- * Author(s) : William Fulton
- *             Neil Cawse
- *
- * Copyright (C) 1999-2002.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.
+ * C# language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.63 2005/05/27 22:40:40 wsfulton Exp $";
+char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.81 2006/03/06 22:50:56 wsfulton Exp $";
 
-#include <limits.h> // for INT_MAX
 #include "swigmod.h"
+#include <limits.h> // for INT_MAX
+#include "cparse.h"
 #include <ctype.h>
 
 
@@ -28,6 +26,7 @@ class CSHARP : public Language {
   File   *f_header;
   File   *f_wrappers;
   File   *f_init;
+  List   *filenames_list;
 
   bool   proxy_flag; // Flag for generating proxy classes
   bool   native_function_flag;     // Flag for when wrapping a native function
@@ -36,6 +35,7 @@ class CSHARP : public Language {
   bool   variable_wrapper_flag; // Flag for when wrapping a nonstatic member variable
   bool   wrapping_member_flag; // Flag for when wrapping a member variable/enum/const
   bool   global_variable_flag; // Flag for when wrapping a global variable
+  bool   old_variable_names;      // Flag for old style variable names in the intermediary class
   bool   generate_property_declaration_flag; // Flag for generating properties
 
   String *imclass_name;  // intermediary class name
@@ -81,6 +81,7 @@ class CSHARP : public Language {
     f_header(NULL),
     f_wrappers(NULL),
     f_init(NULL),
+    filenames_list(NULL),
 
     proxy_flag(true),
     native_function_flag(false),
@@ -89,6 +90,7 @@ class CSHARP : public Language {
     variable_wrapper_flag(false),
     wrapping_member_flag(false),
     global_variable_flag(false),
+    old_variable_names (false),
     generate_property_declaration_flag(false),
 
     imclass_name(NULL),
@@ -170,6 +172,9 @@ class CSHARP : public Language {
         } else if ((strcmp(argv[i],"-noproxy") == 0)) {
           Swig_mark_arg(i);
           proxy_flag = false;
+        } else if (strcmp(argv[i],"-oldvarnames") == 0) {
+          Swig_mark_arg(i);
+	  old_variable_names = true;
         } else if (strcmp(argv[i],"-help") == 0) {
           Printf(stdout,"%s\n", usage);
         }
@@ -205,7 +210,7 @@ class CSHARP : public Language {
 
     f_runtime = NewFile(outfile,"w");
     if (!f_runtime) {
-      Printf(stderr,"Unable to open %s\n", outfile);
+      FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
     f_init = NewString("");
@@ -219,6 +224,7 @@ class CSHARP : public Language {
     Swig_register_filebyname("init",f_init);
 
     swig_types_hash = NewHash();
+    filenames_list = NewList();
 
     // Make the intermediary class and module class names. The intermediary class name can be set in the module directive.
     if (!imclass_name) {
@@ -256,9 +262,10 @@ class CSHARP : public Language {
 
     Printf(wrapper_name, "CSharp_%%f", imclass_name);
     Swig_name_register((char*)"wrapper", Char(wrapper_name));
-    Swig_name_register((char*)"set", (char*)"set_%v");
-    Swig_name_register((char*)"get", (char*)"get_%v");
-    Swig_name_register((char*)"member", (char*)"%c_%m");
+    if (old_variable_names) {
+      Swig_name_register((char*)"set", (char*)"set_%v");
+      Swig_name_register((char*)"get", (char*)"get_%v");
+    }
 
     Delete(wrapper_name);
 
@@ -274,9 +281,10 @@ class CSHARP : public Language {
       String *filen = NewStringf("%s%s.cs", SWIG_output_directory(), imclass_name);
       File *f_im = NewFile(filen,"w");
       if(!f_im) {
-        Printf(stderr,"Unable to open %s\n", filen);
+        FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the intermediary class file
@@ -300,6 +308,7 @@ class CSHARP : public Language {
 
       // Add the intermediary class methods
       Replaceall(imclass_class_code, "$module", module_class_name);
+      Replaceall(imclass_class_code, "$imclassname", imclass_name);
       Replaceall(imclass_class_code, "$dllimport", dllimport);
       Printv(f_im, imclass_class_code, NIL);
       Printv(f_im, imclass_cppcasts_code, NIL);
@@ -315,9 +324,10 @@ class CSHARP : public Language {
       String *filen = NewStringf("%s%s.cs", SWIG_output_directory(), module_class_name);
       File *f_module = NewFile(filen,"w");
       if(!f_module) {
-        Printf(stderr,"Unable to open %s\n", filen);
+        FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the module class file
@@ -342,6 +352,9 @@ class CSHARP : public Language {
       Replaceall(module_class_code, "$module", module_class_name);
       Replaceall(module_class_constants_code, "$module", module_class_name);
 
+      Replaceall(module_class_code, "$imclassname", imclass_name);
+      Replaceall(module_class_constants_code, "$imclassname", imclass_name);
+
       Replaceall(module_class_code, "$dllimport", dllimport);
       Replaceall(module_class_constants_code, "$dllimport", dllimport);
 
@@ -370,7 +383,27 @@ class CSHARP : public Language {
       emitTypeWrapperClass(swig_type.key, swig_type.item);
     }
 
+    // Check for overwriting file problems on filesystems that are case insensitive
+    Iterator it1;
+    Iterator it2;
+    for (it1 = First(filenames_list); it1.item; it1 = Next(it1)) {
+      String *item1_lower = Swig_string_lower(it1.item);
+      for (it2 = Next(it1); it2.item; it2 = Next(it2)) {
+        String *item2_lower = Swig_string_lower(it2.item);
+        if (it1.item && it2.item) {
+          if (Strcmp(item1_lower, item2_lower) == 0) {
+            Swig_warning(WARN_LANG_PORTABILITY_FILENAME, input_file, line_number, 
+              "Portability warning: File %s will be overwritten by %s on case insensitive filesystems such as "
+              "Windows' FAT32 and NTFS unless the class/module name is renamed\n", it1.item, it2.item);
+          }
+        }
+        Delete(item2_lower);
+      }
+      Delete(item1_lower);
+    }
+
     Delete(swig_types_hash); swig_types_hash = NULL;
+    Delete(filenames_list); filenames_list = NULL;
     Delete(imclass_name); imclass_name = NULL;
     Delete(imclass_class_code); imclass_class_code = NULL;
     Delete(proxy_class_def); proxy_class_def = NULL;
@@ -505,7 +538,7 @@ class CSHARP : public Language {
 
     is_void_return = (Cmp(c_return_type, "void") == 0);
     if (!is_void_return)
-      Wrapper_add_localv(f,"jresult", c_return_type, "jresult = 0",NIL);
+      Wrapper_add_localv(f,"jresult", c_return_type, "jresult",NIL);
 
     Printv(f->def, " SWIGEXPORT ", c_return_type, " SWIGSTDCALL ", wname, "(", NIL);
 
@@ -567,7 +600,7 @@ class CSHARP : public Language {
       /* Get the intermediary class parameter types of the parameter */
       if ((tm = Getattr(p,"tmap:imtype"))) {
         String *inattributes = Getattr(p,"tmap:imtype:inattributes");
-        Printf(im_param_type, "%s%s", inattributes ? inattributes : empty_string, tm);
+        Printf(im_param_type, "%s%s", inattributes ? (const String *)inattributes : empty_string, tm);
       } else {
         Swig_warning(WARN_CSHARP_TYPEMAP_CSTYPE_UNDEF, input_file, line_number, 
             "No imtype typemap defined for %s\n", SwigType_str(pt,0));
@@ -662,7 +695,7 @@ class CSHARP : public Language {
       Swig_save("functionWrapper",n,"wrap:action",NIL);
 
       // below based on Swig_VargetToFunction()
-      SwigType *ty = Swig_wrapped_var_type(Getattr(n,"type"));
+      SwigType *ty = Swig_wrapped_var_type(Getattr(n,"type"), use_naturalvar_mode(n));
       Setattr(n,"wrap:action", NewStringf("result = (%s) %s;\n", SwigType_lstr(ty,0), Getattr(n, "value")));
     }
 
@@ -674,6 +707,7 @@ class CSHARP : public Language {
       Swig_restore(n);
 
     /* Return value if necessary  */
+    String *null_attribute = 0;
     if(!native_function_flag) {
       if ((tm = Swig_typemap_lookup_new("out",n,"result",0))) {
         canThrow(n, "out", n);
@@ -681,6 +715,7 @@ class CSHARP : public Language {
         Replaceall(tm,"$target", "jresult"); /* deprecated */
         Replaceall(tm,"$result","jresult");
         Printf(f->code,"%s", tm);
+        null_attribute = Getattr(n, "tmap:out:null");
         if (Len(tm))
           Printf(f->code,"\n");
       } else {
@@ -696,7 +731,7 @@ class CSHARP : public Language {
     Printv(f->code,cleanup,NIL);
 
     /* Look to see if there is any newfree cleanup code */
-    if (Getattr(n,"feature:new")) {
+    if (GetFlag(n,"feature:new")) {
       if ((tm = Swig_typemap_lookup_new("newfree",n,"result",0))) {
         canThrow(n, "newfree", n);
         Replaceall(tm,"$source","result"); /* deprecated */
@@ -731,13 +766,10 @@ class CSHARP : public Language {
       Setattr(n,"csharp:canthrow","1");
     }
 
-    /* Exception macro modification */
-    Replaceall(f->code, "SWIG_exception(", "SWIG_exception($null, ");
-
-    if(!is_void_return)
+    if(!null_attribute)
       Replaceall(f->code,"$null","0");
     else
-      Replaceall(f->code,"$null","");
+      Replaceall(f->code,"$null",null_attribute);
 
     /* Dump the function out */
     if(!native_function_flag) {
@@ -917,9 +949,10 @@ class CSHARP : public Language {
           String *filen = NewStringf("%s%s.cs", SWIG_output_directory(), symname);
           File *f_enum = NewFile(filen,"w");
           if(!f_enum) {
-            Printf(stderr,"Unable to open %s\n", filen);
+            FileErrorDisplay(filen);
             SWIG_exit(EXIT_FAILURE);
           } 
+          Append(filenames_list, Copy(filen));
           Delete(filen); filen = NULL;
 
           // Start writing out the enum file
@@ -1000,8 +1033,7 @@ class CSHARP : public Language {
         Delete(typemap_lookup_type); typemap_lookup_type = NULL;
 
         // The %csconst feature determines how the constant value is obtained
-        String *const_feature = Getattr(n,"feature:cs:const");
-        bool const_feature_flag = const_feature && Cmp(const_feature, "0") != 0;
+        int const_feature_flag = GetFlag(n,"feature:cs:const");
 
         if ((enum_feature == TypesafeEnum) && Getattr(parentNode(n),"sym:name") && !Getattr(parentNode(n),"unnamedinstance")) {
           // Wrap (non-anonymouse) enum using the typesafe enum pattern
@@ -1060,8 +1092,7 @@ class CSHARP : public Language {
     bool is_enum_item = (Cmp(nodeType(n), "enumitem") == 0);
 
     // The %csconst feature determines how the constant value is obtained
-    String *const_feature = Getattr(n,"feature:cs:const");
-    bool const_feature_flag = const_feature && Cmp(const_feature, "0") != 0;
+    int const_feature_flag = GetFlag(n,"feature:cs:const");
 
     /* Adjust the enum type for the Swig_typemap_lookup.
      * We want the same jstype typemap for all the enum items so we use the enum type (parent node). */
@@ -1125,6 +1156,7 @@ class CSHARP : public Language {
         Printf(constants_code, "%s.%s();\n", imclass_name, Swig_name_get(symname));
 
       // Each constant and enum value is wrapped with a separate PInvoke function call
+      SetFlag(n,"feature:immutable");
       enum_constant_flag = true;
       variableWrapper(n);
       enum_constant_flag = false;
@@ -1157,6 +1189,7 @@ class CSHARP : public Language {
   virtual int insertDirective(Node *n) {
     String *code = Getattr(n,"code");
     Replaceall(code, "$module", module_class_name);
+    Replaceall(code, "$imclassname", imclass_name);
     Replaceall(code, "$dllimport", dllimport);
     return Language::insertDirective(n);
   }
@@ -1251,15 +1284,27 @@ class CSHARP : public Language {
     List *baselist = Getattr(n,"bases");
     if (baselist) {
       Iterator base = First(baselist);
-      c_baseclassname = Getattr(base.item,"name");
-      baseclass = Copy(getProxyName(c_baseclassname));
-      if (baseclass){
-        c_baseclass = SwigType_namestr(Getattr(base.item,"name"));
+      while(base.item && GetFlag(base.item,"feature:ignore")) {
+	base = Next(base);
       }
-      base = Next(base);
       if (base.item) {
-        Swig_warning(WARN_CSHARP_MULTIPLE_INHERITANCE, input_file, line_number, 
-            "Warning for %s proxy: Base %s ignored. Multiple inheritance is not supported in C#.\n", typemap_lookup_type, Getattr(base.item,"name"));
+	c_baseclassname = Getattr(base.item,"name");
+	baseclass = Copy(getProxyName(c_baseclassname));
+	if (baseclass)
+	  c_baseclass = SwigType_namestr(Getattr(base.item,"name"));
+	base = Next(base);
+	/* Warn about multiple inheritance for additional base class(es) */
+	while (base.item) {
+	  if (GetFlag(base.item,"feature:ignore")) {
+	    base = Next(base);
+	    continue;
+	  }
+	  String *proxyclassname = SwigType_str(Getattr(n,"classtypeobj"),0);
+	  String *baseclassname = SwigType_str(Getattr(base.item,"name"),0);
+	  Swig_warning(WARN_CSHARP_MULTIPLE_INHERITANCE, input_file, line_number, 
+	      "Warning for %s proxy: Base %s ignored. Multiple inheritance is not supported in C#.\n", proxyclassname, baseclassname);
+	  base = Next(base);
+	}
       }
     }
 
@@ -1357,6 +1402,9 @@ class CSHARP : public Language {
     Replaceall(proxy_class_def,  "$module", module_class_name);
     Replaceall(proxy_class_code, "$module", module_class_name);
 
+    Replaceall(proxy_class_def,  "$imclassname", imclass_name);
+    Replaceall(proxy_class_code, "$imclassname", imclass_name);
+
     Replaceall(proxy_class_def,  "$dllimport", dllimport);
     Replaceall(proxy_class_code, "$dllimport", dllimport);
 
@@ -1407,9 +1455,10 @@ class CSHARP : public Language {
       String *filen = NewStringf("%s%s.cs", SWIG_output_directory(), proxy_class_name);
       f_proxy = NewFile(filen,"w");
       if(!f_proxy) {
-        Printf(stderr, "Unable to create proxy class file: %s\n", filen);
+        FileErrorDisplay(filen);
         SWIG_exit(EXIT_FAILURE);
       }
+      Append(filenames_list, Copy(filen));
       Delete(filen); filen = NULL;
 
       // Start writing out the proxy class file
@@ -1434,7 +1483,9 @@ class CSHARP : public Language {
       Replaceall(proxy_class_def, "$module", module_class_name);
       Replaceall(proxy_class_code, "$module", module_class_name);
       Replaceall(proxy_class_constants_code, "$module", module_class_name);
-
+      Replaceall(proxy_class_def, "$imclassname", imclass_name);
+      Replaceall(proxy_class_code, "$imclassname", imclass_name);
+      Replaceall(proxy_class_constants_code, "$imclassname", imclass_name);
       Replaceall(proxy_class_def, "$dllimport", dllimport);
       Replaceall(proxy_class_code, "$dllimport", dllimport);
       Replaceall(proxy_class_constants_code, "$dllimport", dllimport);
@@ -1620,7 +1671,7 @@ class CSHARP : public Language {
         if ((tm = Getattr(p,"tmap:cstype"))) {
           substituteClassname(pt, tm);
           String *inattributes = Getattr(p,"tmap:cstype:inattributes");
-          Printf(param_type, "%s%s", inattributes ? inattributes : empty_string, tm);
+          Printf(param_type, "%s%s", inattributes ? (const String *)inattributes : empty_string, tm);
         } else {
           Swig_warning(WARN_CSHARP_TYPEMAP_CSWTYPE_UNDEF, input_file, line_number, 
               "No cstype typemap defined for %s\n", SwigType_str(pt,0));
@@ -1658,7 +1709,7 @@ class CSHARP : public Language {
 
     // Transform return type used in PInvoke function (in intermediary class) to type used in C# wrapper function (in proxy class)
     if ((tm = Swig_typemap_lookup_new("csout",n,"",0))) {
-      if (Getattr(n,"feature:new"))
+      if (GetFlag(n,"feature:new"))
         Replaceall(tm,"$owner","true");
       else
         Replaceall(tm,"$owner","false");
@@ -1714,7 +1765,7 @@ class CSHARP : public Language {
       } else {
         // Getter method
         if ((tm = Swig_typemap_lookup_new("csvarout",n,"",0))) {
-          if (Getattr(n,"feature:new"))
+          if (GetFlag(n,"feature:new"))
             Replaceall(tm,"$owner","true");
           else
             Replaceall(tm,"$owner","false");
@@ -1800,7 +1851,7 @@ class CSHARP : public Language {
         if ((tm = Getattr(p,"tmap:cstype"))) {
           substituteClassname(pt, tm);
           String *inattributes = Getattr(p,"tmap:cstype:inattributes");
-          Printf(param_type, "%s%s", inattributes ? inattributes : empty_string, tm);
+          Printf(param_type, "%s%s", inattributes ? (const String *)inattributes : empty_string, tm);
         } else {
           Swig_warning(WARN_CSHARP_TYPEMAP_CSWTYPE_UNDEF, input_file, line_number, 
               "No cstype typemap defined for %s\n", SwigType_str(pt,0));
@@ -2028,7 +2079,7 @@ class CSHARP : public Language {
       if ((tm = Getattr(p,"tmap:cstype"))) {
         substituteClassname(pt, tm);
         String *inattributes = Getattr(p,"tmap:cstype:inattributes");
-        Printf(param_type, "%s%s", inattributes ? inattributes : empty_string, tm);
+        Printf(param_type, "%s%s", inattributes ? (const String *)inattributes : empty_string, tm);
       } else {
         Swig_warning(WARN_CSHARP_TYPEMAP_CSWTYPE_UNDEF, input_file, line_number, 
             "No cstype typemap defined for %s\n", SwigType_str(pt,0));
@@ -2065,7 +2116,7 @@ class CSHARP : public Language {
 
     // Transform return type used in PInvoke function (in intermediary class) to type used in C# wrapper function (in module class)
     if ((tm = Swig_typemap_lookup_new("csout",n,"",0))) {
-      if (Getattr(n,"feature:new"))
+      if (GetFlag(n,"feature:new"))
         Replaceall(tm,"$owner","true");
       else
         Replaceall(tm,"$owner","false");
@@ -2120,7 +2171,7 @@ class CSHARP : public Language {
       } else {
         // Getter method
         if ((tm = Swig_typemap_lookup_new("csvarout",n,"",0))) {
-          if (Getattr(n,"feature:new"))
+          if (GetFlag(n,"feature:new"))
             Replaceall(tm,"$owner","true");
           else
             Replaceall(tm,"$owner","false");
@@ -2187,15 +2238,14 @@ class CSHARP : public Language {
 
     if (!value) {
       // The %csconst feature determines how the constant value is obtained
-      String *const_feature = Getattr(n,"feature:cs:const");
-      bool const_feature_flag = const_feature && Cmp(const_feature, "0") != 0;
+      int const_feature_flag = GetFlag(n,"feature:cs:const");
 
       if (const_feature_flag) {
         // Use the C syntax to make a true C# constant and hope that it compiles as C# code
         value = Getattr(n,"enumvalue") ? Copy(Getattr(n,"enumvalue")) : Copy(Getattr(n,"enumvalueex"));
       } else {
         // Get the enumvalue from a PINVOKE call
-        if (!getCurrentClass()) {
+        if (!getCurrentClass() || !cparse_cplusplus) {
           // Strange hack to change the name
           Setattr(n,"name",Getattr(n,"value")); /* for wrapping of enums in a namespace when emit_action is used */
           constantWrapper(n);
@@ -2339,7 +2389,8 @@ class CSHARP : public Language {
           count++;
         plist = nextSibling(plist);
       }
-      arg = (!pn || (count > 1)) ? NewStringf("arg%d",arg_num) : Copy(pn);
+      String *wrn = pn ? Swig_name_warning(p,0,pn,0) : 0;
+      arg = (!pn || (count > 1) || wrn) ? NewStringf("arg%d",arg_num) : Copy(pn);
     }
 
     return arg;
@@ -2354,9 +2405,10 @@ class CSHARP : public Language {
     String *filen = NewStringf("%s%s.cs", SWIG_output_directory(), classname);
     File *f_swigtype = NewFile(filen,"w");
     if(!f_swigtype) {
-      Printf(stderr,"Unable to open %s\n", filen);
+      FileErrorDisplay(filen);
       SWIG_exit(EXIT_FAILURE);
     } 
+    Append(filenames_list, Copy(filen));
     Delete(filen); filen = NULL;
 
     // Start writing out the type wrapper class file
@@ -2402,6 +2454,7 @@ class CSHARP : public Language {
 
     Replaceall(swigtype, "$csclassname", classname);
     Replaceall(swigtype, "$module", module_class_name);
+    Replaceall(swigtype, "$imclassname", imclass_name);
     Replaceall(swigtype, "$dllimport", dllimport);
     Printv(f_swigtype, swigtype, NIL);
 
@@ -2492,5 +2545,6 @@ C# Options (available with -csharp)\n\
      -namespace <nm> - Generate wrappers into C# namespace <nm>\n\
      -noproxy        - Generate the low-level functional interface instead\n\
                        of proxy classes\n\
+     -oldvarnames    - old intermediary method names for variable wrappers\n\
 \n";
 
