@@ -1,25 +1,16 @@
 /* -----------------------------------------------------------------------------
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ *
  * swigmod.h
  *
- *     Main header file for SWIG modules
- *
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1998-2000.  The University of Chicago
- * Copyright (C) 1995-1998.  The University of Utah and The Regents of the
- *                           University of California.
- *
- * See the file LICENSE for information on usage and redistribution.
- *
- * $Header: /cvsroot/swig/SWIG/Source/Modules/swigmod.h,v 1.37 2005/04/01 19:43:47 wsfulton Exp $
+ * Main header file for SWIG modules.
  * ----------------------------------------------------------------------------- */
 
-#ifndef SWIGMOD_H_
-#define SWIGMOD_H_
+/* $Header: /cvsroot/swig/SWIG/Source/Modules/swigmod.h,v 1.44 2006/03/07 22:02:54 wsfulton Exp $ */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#ifndef SWIG_SWIGMOD_H_
+#define SWIG_SWIGMOD_H_
 
 #include "swig.h"
 #include "preprocessor.h"
@@ -111,13 +102,13 @@ protected:
   AccessMode cplus_mode;
 };
 
-/************************************************************************
+/* ----------------------------------------------------------------------------
  * class language:
  *
  * This class defines the functions that need to be supported by the
  * scripting language being used.    The translator calls these virtual
  * functions to output different types of code for different languages.
- *************************************************************************/
+ * ------------------------------------------------------------------------- */
 
 class Language : public Dispatcher {
 public:
@@ -319,8 +310,11 @@ int    emit_isvarargs(ParmList *);
 void   emit_attach_parmmaps(ParmList *, Wrapper *f);
 void   emit_mark_varargs(ParmList *l);
 void   emit_action(Node *n, Wrapper *f);
+int    emit_action_code(Node *n, Wrapper *f, String *action);
 void   Swig_overload_check(Node *n);
 String *Swig_overload_dispatch(Node *n, const String_or_char *fmt, int *);
+String *Swig_overload_dispatch_cast(Node *n, const String_or_char *fmt, int *);
+String *Swig_overload_dispatch_fast(Node *n, const String_or_char *fmt, int *);
 SwigType *cplus_value_type(SwigType *t);
 
 /* directors.cxx start */
@@ -347,9 +341,15 @@ int is_private(Node* n);
 int is_protected(Node* n); 
 int is_member_director(Node* parentnode, Node* member); 
 int is_member_director(Node* member); 
+int use_naturalvar_mode(Node *n);
+
 void Wrapper_virtual_elimination_mode_set(int);
 void Wrapper_director_mode_set(int);
 void Wrapper_director_protected_mode_set(int);
+void Wrapper_fast_dispatch_mode_set(int);
+void Wrapper_cast_dispatch_mode_set(int);
+void Wrapper_naturalvar_mode_set(int);
+
 
 void clean_overloaded(Node *n);
 
