@@ -1,25 +1,21 @@
 /* ----------------------------------------------------------------------------- 
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ *
  * typepass.cxx
  *
- *     This module builds all of the internal type information by collecting
- *     typedef declarations as well as registering classes, structures, and unions.
- *     This information is needed to correctly handle shadow classes and other
- *     advanced features.   This phase of compilation is also used to perform
- *     type-expansion.  All types are fully qualified with namespace prefixes
- *     and other information needed for compilation.
- *
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1998-2002.  The University of Chicago
- * Copyright (C) 1995-1998.  The University of Utah and The Regents of the
- *                           University of California.
- *
- * See the file LICENSE for information on usage and redistribution.	
+ * This module builds all of the internal type information by collecting
+ * typedef declarations as well as registering classes, structures, and unions.
+ * This information is needed to correctly handle shadow classes and other
+ * advanced features.   This phase of compilation is also used to perform
+ * type-expansion.  All types are fully qualified with namespace prefixes
+ * and other information needed for compilation.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.24 2005/05/25 21:23:15 wsfulton Exp $";
+char cvsroot_typepass_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/typepass.cxx,v 1.35 2006/03/06 22:50:59 wsfulton Exp $";
 
 #include "swigmod.h"
+#include "cparse.h"
 
 struct normal_node {
     Symtab *symtab;
@@ -131,7 +127,7 @@ class TypePass : private Dispatcher {
       
     if (first == cls) return;  /* The Marcelo check */
     if (!cls) cls = first;
-
+    List *alist = 0;
     List *ilist = Getattr(cls,bases);
     if (!ilist) {
       List *nlist = Getattr(cls,baselist);
@@ -184,7 +180,7 @@ class TypePass : private Dispatcher {
 		bcls = 0;
 	      } else {
 		if (Getattr(bcls,"typepass:visit")) {
-		  if (!ilist) ilist = NewList();
+		  if (!ilist) ilist = alist = NewList();
 		  Append(ilist,bcls);
 		} else {
 		  Swig_warning(WARN_TYPE_UNDEFINED_CLASS,Getfile(cls),Getline(cls),"Base class '%s' undefined.\n", bname);
@@ -214,6 +210,8 @@ class TypePass : private Dispatcher {
 	Setattr(cls,bases,ilist);
       }
     }
+    if (alist) Delete(alist);
+
     if (!ilist) return;
     int len = Len(ilist);
     int i;
@@ -233,14 +231,22 @@ class TypePass : private Dispatcher {
 	SwigType_inherit_scope(scopes);
       }
       /* Set up inheritance in the symbol table */
-      Symtab *s = Swig_symbol_current();
       Symtab *st = Getattr(cls,"symtab");
+      Symtab *bst = Getattr(bclass,"symtab");
+      if (st == bst) {
+	Swig_warning(WARN_PARSE_REC_INHERITANCE, Getfile(cls), Getline(cls), 
+		     "Recursive scope inheritance of '%s'.\n", HashGetAttr(cls,k_name));
+	continue;
+      }
+      Symtab *s = Swig_symbol_current();
       Swig_symbol_setscope(st);
-      Swig_symbol_inherit(Getattr(bclass,"symtab"));
+      Swig_symbol_inherit(bst);
       Swig_symbol_setscope(s);
 
       /* Recursively hit base classes */
-      String *newcast = NewStringf("(%s *)%s", SwigType_namestr(Getattr(bclass,"name")), cast);
+      String *namestr = SwigType_namestr(Getattr(bclass,"name"));
+      String *newcast = NewStringf("(%s *)%s", namestr, cast);
+      Delete(namestr);
       cplus_inherit_types_impl(first,bclass,clsname,bases,baselist,ispublic,newcast);
       Delete(newcast);
     }
@@ -267,9 +273,8 @@ class TypePass : private Dispatcher {
     append_list(allbases,Getattr(cls,"privatebases"));
     if (Len(allbases)) {
       Setattr(cls,"allbases",allbases);
-    } else {
-      Delete(allbases);
     }
+    Delete(allbases);
   }
 
   /* ------------------------------------------------------------
@@ -350,9 +355,9 @@ class TypePass : private Dispatcher {
 	// We need to fully resolve the name to make templates work correctly */
 	Node *cn;
 	fname = SwigType_typedef_resolve_all(name);
-	if (Strcmp(fname,name) != 0) {
-	  cn = Swig_symbol_clookup_local(fname,0);
-	  if ((!cn) || (Strcmp(nodeType(cn),"template") == 0)
+	if (Strcmp(fname,name) != 0 && (cn = Swig_symbol_clookup_local(fname,0))) {
+	  if ((n == cn) 
+	      || (Strcmp(nodeType(cn),"template") == 0)
 	      || (Getattr(cn,"feature:onlychildren") != 0)
 	      || (Getattr(n,"feature:onlychildren") != 0)) {
 	    Swig_symbol_cadd(fname,n);
@@ -434,6 +439,7 @@ class TypePass : private Dispatcher {
     /* If in a namespace, patch the class name */
     if (nname) {
       Setattr(n,"name",nname);
+      Delete(nname);
     }
     return SWIG_OK;
   }
@@ -571,7 +577,7 @@ class TypePass : private Dispatcher {
       SwigType *t = Copy(ty);
       {
 	/* If the typename is qualified, make sure the scopename is fully qualified when making a typedef */
-	if (Swig_scopename_check(t)) {
+	if (Swig_scopename_check(t) && strncmp(Char(t),"::",2)) {
 	  String *base, *prefix, *qprefix;
 	  base = Swig_scopename_last(t);
 	  prefix = Swig_scopename_prefix(t);
@@ -722,7 +728,7 @@ class TypePass : private Dispatcher {
     if (!value) value = name;
     if (Strcmp(value,name) == 0) {
       String *new_value;
-      if ((nsname) || (inclass)) {
+      if (((nsname) || (inclass)) && cparse_cplusplus ) {
 	new_value = NewStringf("%s::%s", SwigType_namestr(Swig_symbol_qualified(n)), value);
       } else {
 	new_value = NewString(value);
@@ -774,9 +780,17 @@ class TypePass : private Dispatcher {
     } else {
       Node *ns;
       /* using id */
-
-      if (Getattr(n,"sym:symtab")) {
-	ns = Swig_symbol_clookup(Getattr(n,"uname"), Getattr(n,"sym:symtab"));
+      Symtab *stab = Getattr(n,"sym:symtab");
+      if (stab) {
+	String *uname = Getattr(n,"uname");
+	ns = Swig_symbol_clookup(uname, stab);
+	if (!ns && SwigType_istemplate(uname)) {
+	  String *tmp = Swig_symbol_template_deftype(uname, 0);
+	  if (!Equal(tmp, uname)) {
+	    ns = Swig_symbol_clookup(tmp, stab);
+	  }	  
+	  Delete(tmp);
+	}
       } else {
 	ns = 0;
       }
@@ -797,7 +811,7 @@ class TypePass : private Dispatcher {
 	    SwigType_typedef_using(uname);
 	  } else {
 	    /* A normal C declaration. */
-	    if ((inclass) && (!Getattr(n,"feature:ignore")) && (Getattr(n,"sym:name"))) {
+	    if ((inclass) && (!GetFlag(n,"feature:ignore")) && (Getattr(n,"sym:name"))) {
 	      Node *c = ns;
 	      Node *unodes = 0, *last_unodes = 0;
 	      int   ccount = 0;
@@ -808,7 +822,7 @@ class TypePass : private Dispatcher {
 			|| checkAttribute(c,"storage","typedef")
 			|| checkAttribute(c,"storage","friend")
 			|| (Getattr(c,"feature:extend") && !Getattr(c,"code"))
-			|| Getattr(c,"feature:ignore"))) {
+			|| GetFlag(c,"feature:ignore"))) {
 			
 		    String *csymname = Getattr(c,"sym:name");
 		    if (!csymname || (Strcmp(csymname,symname) == 0)) {
@@ -834,8 +848,25 @@ class TypePass : private Dispatcher {
 		      Delattr(nn,"access"); // access might be different from the method in the base class
 		      if (!Getattr(nn,"sym:name")) Setattr(nn,"sym:name", symname);
 
-		      if (!Getattr(nn,"feature:ignore")) {
-			Setattr(nn,"parms",CopyParmList(Getattr(c,"parms")));
+		      if (!GetFlag(nn,"feature:ignore")) {
+			ParmList *parms = CopyParmList(Getattr(c,"parms"));
+			int is_pointer = SwigType_ispointer_return(Getattr(nn,"decl"));
+			int is_void = checkAttribute(nn,"type", "void") && !is_pointer;
+			Setattr(nn,"parms",parms);
+			Delete(parms);
+			if (Getattr(n,"feature:extend")) {
+			  String *ucode = is_void ? NewStringf("{ self->%s(",Getattr(n,"uname")) :
+			    NewStringf("{ return self->%s(",Getattr(n,"uname"));
+
+			  for (ParmList *p = parms; p;) {
+			    StringAppend(ucode,HashGetAttr(p,k_name));
+			    p = nextSibling(p);
+			    if (p) StringAppend(ucode,",");
+			  }
+			  StringAppend(ucode,"); }");
+			  Setattr(nn,"code",ucode);
+			  Delete(ucode);
+			}
 			ParmList *throw_parm_list = Getattr(c,"throws");
 			if (throw_parm_list)
 			  Setattr(nn,"throws",CopyParmList(throw_parm_list));
