@@ -1,20 +1,18 @@
 /* ----------------------------------------------------------------------------- 
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ *
  * fragment.c
  *
- *     This file manages named code fragments.  Code fragments are typically
- *     used to hold helper-code that may or may not be included in the wrapper
- *     file (depending on what features are actually used in the interface).
+ * This file manages named code fragments.  Code fragments are typically
+ * used to hold helper-code that may or may not be included in the wrapper
+ * file (depending on what features are actually used in the interface).
  *
- *     By using fragments, it's possible to greatly reduce the amount of
- *     wrapper code and to generate cleaner wrapper files. 
- * 
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.	
+ * By using fragments, it's possible to greatly reduce the amount of
+ * wrapper code and to generate cleaner wrapper files. 
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_fragment_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/fragment.c,v 1.14 2006/02/03 07:33:28 marcelomatus Exp $";
+char cvsroot_fragment_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/fragment.c,v 1.15 2006/03/06 22:51:00 wsfulton Exp $";
 
 #include "swig.h"
 #include "swigkeys.h"
