@@ -1,16 +1,14 @@
 /* -----------------------------------------------------------------------------
+ * See the LICENSE file for information on copyright, usage and redistribution
+ * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ *
  * stype.c
  *
- *     This file provides general support for datatypes that are encoded in
- *     the form of simple strings.
- *
- * Author(s) : David Beazley (beazley@cs.uchicago.edu)
- *
- * Copyright (C) 1999-2000.  The University of Chicago
- * See the file LICENSE for information on usage and redistribution.
+ * This file provides general support for datatypes that are encoded in
+ * the form of simple strings.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_stype_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/stype.c,v 1.58 2005/03/28 19:38:37 wsfulton Exp $";
+char cvsroot_stype_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/stype.c,v 1.75 2006/03/20 10:47:39 marcelomatus Exp $";
 
 #include "swig.h"
 #include "cparse.h"
@@ -146,7 +144,7 @@ SwigType *NewSwigType(int t) {
   default :
     break;
   }
-  return NewString("");
+  return NewStringEmpty();
 }
 
 /* -----------------------------------------------------------------------------
@@ -325,11 +323,11 @@ static
 void SwigType_add_default(String *def, SwigType *nr)
 {
   if (Strcmp(nr,"SWIGTYPE") == 0) {
-    Append(def,"SWIGTYPE");
+    StringAppend(def,"SWIGTYPE");
   } else {
     String *q = SwigType_isqualifier(nr) ? SwigType_pop(nr) : 0;
-    if (q && Strstr(nr,"SWIGTYPE")) {
-      Append(def, nr);
+    if (q && strstr(Char(nr),"SWIGTYPE")) {
+      StringAppend(def, nr);
     } else {
       String *nd = SwigType_default(nr);
       if (nd) {
@@ -343,10 +341,10 @@ void SwigType_add_default(String *def, SwigType *nr)
 	    Delete(nd);
 	  }
 	}
-	Append(def,bdef);
+	StringAppend(def,bdef);
 	Delete(bdef);
       } else {
-	Append(def,nr);
+	StringAppend(def,nr);
       }
     }
     Delete(q);
@@ -357,13 +355,13 @@ void SwigType_add_default(String *def, SwigType *nr)
 SwigType *SwigType_default(SwigType *t) {
   String *r1, *def;
   String *r = 0;
+  char *cr;
 
 #ifdef SWIG_DEFAULT_CACHE
   if (!default_cache) default_cache = NewHash();
   
   r = Getattr(default_cache,t);
   if (r) {
-    if (Strcmp(r,t) == 0) return 0;
     return Copy(r);
   }
 #endif
@@ -381,41 +379,43 @@ SwigType *SwigType_default(SwigType *t) {
     String *q;
     if (r == t) r = Copy(t);
     q = SwigType_pop(r);
-    if (Strstr(r,"SWIGTYPE")) {
+    if (strstr(Char(r),"SWIGTYPE")) {
       Delete(q);
       def = r;
       return def;
     }
     Delete(q);
   }
-  if (Strcmp(r,"p.SWIGTYPE") == 0) {
+  cr = Char(r);
+  if (strcmp(cr,"p.SWIGTYPE") == 0) {
     def = NewString("SWIGTYPE");
   } else if (SwigType_ispointer(r)) {
 #ifdef SWIG_NEW_TYPE_DEFAULT
     SwigType *nr = Copy(r);
     SwigType_del_pointer(nr);
-    def = NewStringf("p.");
+    def = SwigType_isfunction(nr) ? 
+      NewStringEmpty() : NewString("p.");
     SwigType_add_default(def, nr);
     Delete(nr);
 #else
     def = NewString("p.SWIGTYPE");
 #endif
-  } else if (Strcmp(r,"r.SWIGTYPE") == 0) {
+  } else if (strcmp(cr,"r.SWIGTYPE") == 0) {
     def = NewString("SWIGTYPE");
   } else if (SwigType_isreference(r)) {
 #ifdef SWIG_NEW_TYPE_DEFAULT
     SwigType *nr = Copy(r);
     SwigType_del_reference(nr);
-    def = NewStringf("r.");
+    def = NewString("r.");
     SwigType_add_default(def, nr);
     Delete(nr);
 #else
     def = NewString("r.SWIGTYPE");
 #endif
   } else if (SwigType_isarray(r)) {
-    if (Strcmp(r,"a().SWIGTYPE") == 0) {
+    if (strcmp(cr,"a().SWIGTYPE") == 0) {
       def = NewString("p.SWIGTYPE");
-    } else if (Strcmp(r,"a(ANY).SWIGTYPE") == 0) {
+    } else if (strcmp(cr,"a(ANY).SWIGTYPE") == 0) {
       def = NewString("a().SWIGTYPE");
     } else {
       int i, empty = 0;
@@ -437,36 +437,47 @@ SwigType *SwigType_default(SwigType *t) {
 #ifdef SWIG_NEW_TYPE_DEFAULT
       SwigType_del_array(nr);
       SwigType_add_default(def, nr);
-      Delete(nr);
 #else
-      Append(def,"SWIGTYPE");
+      StringAppend(def,"SWIGTYPE");
 #endif
+      Delete(nr);
     }
   } else if (SwigType_ismemberpointer(r)) {
-    if (Strcmp(r,"m(CLASS).SWIGTYPE") == 0) {
+    if (strcmp(cr,"m(CLASS).SWIGTYPE") == 0) {
       def = NewString("p.SWIGTYPE");
     } else {
       def = NewString("m(CLASS).SWIGTYPE");
     }
   } else if (SwigType_isenum(r)) {
-    if (Strcmp(r,"enum SWIGTYPE") == 0) {
+    if (strcmp(cr,"enum SWIGTYPE") == 0) {
       def = NewString("SWIGTYPE");
     } else {
       def = NewString("enum SWIGTYPE");
     }
+  } else if (SwigType_isfunction(r)) {
+    if (strcmp(cr,"f(ANY).SWIGTYPE") == 0) {
+      def = NewString("p.SWIGTYPE");
+    } else {
+      def = NewString("p.f(ANY).SWIGTYPE");
+    }
   } else {
     def = NewString("SWIGTYPE");
   }
   if (r != t) Delete(r);
-#ifdef SWIG_DEFAULT_CACHE
-  /* The cache produces strange results, see enum_template.i case */
-  Setattr(default_cache,t,Copy(def)); 
-#endif
-  if (Strcmp(def,t) == 0) {
+  if (StringEqual(def,t)) {
     Delete(def);
     def = 0;
   }
 
+#ifdef SWIG_DEFAULT_CACHE
+  /* The cache produces strange results, see enum_template.i case */
+  if (def) {
+    String *cdef  = Copy(def);
+    Setattr(default_cache,t, cdef); 
+    Delete(cdef);
+  }
+#endif
+
   /* Printf(stderr,"type : def %s : %s\n", t, def);  */
 
   return def;
@@ -483,44 +494,30 @@ SwigType_namestr(const SwigType *t) {
   String *r;
   String *suffix;
   List   *p;
-  char   tmp[256];
-  char   *c, *d, *e;
   int     i, sz;
+  char   *d = Char(t);
+  char   *c = strstr(d,"<(");
 
-  if (!SwigType_istemplate(t)) return NewString(t);
+  if (!c || !strstr(c+2,")>")) return NewString(t);
 
-  c = Strstr(t,"<(");
-
-  d = Char(t);
-  e = tmp;
-  while (d != c) {
-    *(e++) = *(d++);
-  }
-  *e = 0;
-  r = NewString(tmp);
+  r = NewStringWithSize(d, c - d);
+  if (*(c - 1) == '<')  Putc(' ',r);
   Putc('<',r);
   
-  p = SwigType_parmlist(t);
+  p = SwigType_parmlist(c+1);
   sz = Len(p);
   for (i = 0; i < sz; i++) {
     String *str = SwigType_str(Getitem(p,i),0);
-    Append(r,str);
+    StringAppend(r,str);
     if ((i+1) < sz) Putc(',',r);
     Delete(str);
   }
   Putc(' ',r);
   Putc('>',r);
   suffix = SwigType_templatesuffix(t);
-  Append(r,suffix);
+  StringAppend(r,suffix);
   Delete(suffix);
   Delete(p);
-#if 0
-  if (SwigType_istemplate(r)) {
-    SwigType *rr = SwigType_namestr(r);
-    Delete(r);
-    return rr;
-  }
-#endif
   return r;
 }
 
@@ -539,9 +536,9 @@ SwigType_str(SwigType *s, const String_or_char *id)
   int nelements, i;
 
   if (id) {
-    result = NewString(Char(id));
+    result = NewString(id);
   } else {
-    result = NewString("");
+    result = NewStringEmpty();
   }
 
   elements = SwigType_split(s);
@@ -567,7 +564,7 @@ SwigType_str(SwigType *s, const String_or_char *id)
       Insert(result,0,"*");
       if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	Insert(result,0,"(");
-	Append(result,")");
+	StringAppend(result,")");
       }
     } else if (SwigType_ismemberpointer(element)) {
       String *q;
@@ -576,7 +573,7 @@ SwigType_str(SwigType *s, const String_or_char *id)
       Insert(result,0,q);
       if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	Insert(result,0,"(");
-	Append(result,")");
+	StringAppend(result,")");
       }
       Delete(q);
     }
@@ -584,30 +581,30 @@ SwigType_str(SwigType *s, const String_or_char *id)
       Insert(result,0,"&");
       if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	Insert(result,0,"(");
-	Append(result,")");
+	StringAppend(result,")");
       }
     }  else if (SwigType_isarray(element)) {
       DOH *size;
-      Append(result,"[");
+      StringAppend(result,"[");
       size = SwigType_parm(element);
-      Append(result,size);
-      Append(result,"]");
+      StringAppend(result,size);
+      StringAppend(result,"]");
       Delete(size);
     } else if (SwigType_isfunction(element)) {
       DOH *parms, *p;
       int j, plen;
-      Append(result,"(");
+      StringAppend(result,"(");
       parms = SwigType_parmlist(element);
       plen = Len(parms);
       for (j = 0; j < plen; j++) {
 	p = SwigType_str(Getitem(parms,j),0);
-	Append(result,p);
-	if (j < (plen-1)) Append(result,",");
+	StringAppend(result,p);
+	if (j < (plen-1)) StringAppend(result,",");
       }
-      Append(result,")");
+      StringAppend(result,")");
       Delete(parms);
     } else {
-      if (Strcmp(element,"v(...)") == 0) {
+      if (strcmp(Char(element),"v(...)") == 0) {
 	Insert(result,0,"...");
       } else {
 	String *bs = SwigType_namestr(element);
@@ -639,7 +636,7 @@ SwigType_ltype(SwigType *s) {
   int firstarray = 1;
   int notypeconv = 0;
 
-  result = NewString("");
+  result = NewStringEmpty();
   tc = Copy(s);
   /* Nuke all leading qualifiers */
   while (SwigType_isqualifier(tc)) {
@@ -647,13 +644,22 @@ SwigType_ltype(SwigType *s) {
   }
   if (SwigType_issimple(tc)) {
     /* Resolve any typedef definitions */
-    td = SwigType_typedef_resolve(tc);
-    if (td && (SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
-      /* We need to use the typedef type */
+    SwigType *tt  = Copy(tc);
+    td = 0;
+    while ((td = SwigType_typedef_resolve(tt))) {      
+      if (td && (SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
+	/* We need to use the typedef type */
+	Delete(tt);
+	tt = td;
+	break;
+      } else if (td) {
+	Delete(tt);
+	tt = td;
+      }
+    }
+    if (td) {
       Delete(tc);
-      tc = td;
-    } else if (td) {
-      Delete(td);
+      tc = td;      
     }
   }
   elements = SwigType_split(tc);
@@ -669,34 +675,34 @@ SwigType_ltype(SwigType *s) {
     if (SwigType_isqualifier(element)) {
       /* Do nothing. Ignore */
     } else if (SwigType_ispointer(element)) {
-      Append(result,element);
+      StringAppend(result,element);
       firstarray = 0;
     } else if (SwigType_ismemberpointer(element)) {
-      Append(result,element);
+      StringAppend(result,element);
       firstarray = 0;
     } else if (SwigType_isreference(element)) {
       if (notypeconv) {
-	Append(result,element);
+	StringAppend(result,element);
       } else {
-	Append(result,"p.");
+	StringAppend(result,"p.");
       }
       firstarray = 0;
     } else if (SwigType_isarray(element) && firstarray) {
       if (notypeconv) {
-	Append(result,element);
+	StringAppend(result,element);
       } else {
-	Append(result,"p.");
+	StringAppend(result,"p.");
       }
       firstarray = 0;
     } else if (SwigType_isenum(element)) {
       int anonymous_enum = (Cmp(element,"enum ") == 0); 
       if (notypeconv || !anonymous_enum) {
-	Append(result,element);
+	StringAppend(result,element);
       } else {
-	Append(result,"int");
+	StringAppend(result,"int");
       }
     } else {
-      Append(result,element);
+      StringAppend(result,element);
     }
   }
   Delete(elements);
@@ -749,7 +755,7 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
   int      isreference = 0;
   int      isarray = 0;
 
-  result = NewString("");
+  result = NewStringEmpty();
 
   if (SwigType_isconst(s)) {
     tc = Copy(s);
@@ -759,7 +765,12 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
     rs = s;
   }
 
-  td = SwigType_typedef_resolve(rs);
+  if ((SwigType_isconst(rs) || SwigType_isarray(rs) || SwigType_isreference(rs))) {
+    td = 0;
+  } else {
+    td = SwigType_typedef_resolve(rs);
+  }
+  
   if (td) {
     if ((SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
       elements = SwigType_split(td);
@@ -792,7 +803,7 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
       Insert(result,0,"*");
       if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	Insert(result,0,"(");
-	Append(result,")");
+	StringAppend(result,")");
       }
       firstarray = 0;
     } else  if (SwigType_ismemberpointer(element)) {
@@ -803,26 +814,26 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
       Delete(q);
       if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	Insert(result,0,"(");
-	Append(result,")");
+	StringAppend(result,")");
       }
       firstarray = 0;
     } else if (SwigType_isreference(element)) {
 	Insert(result,0,"&");
 	if ((nextelement) && ((SwigType_isfunction(nextelement) || (SwigType_isarray(nextelement))))) {
 	  Insert(result,0,"(");
-	  Append(result,")");
+	  StringAppend(result,")");
 	}
 	isreference = 1;
     } else if (SwigType_isarray(element)) {
       DOH *size;
       if (firstarray && !isreference) {
-	Append(result,"(*)");
+	StringAppend(result,"(*)");
 	firstarray = 0;
       } else {
-	Append(result,"[");
+	StringAppend(result,"[");
 	size = SwigType_parm(element);
-	Append(result,size);
-	Append(result,"]");
+	StringAppend(result,size);
+	StringAppend(result,"]");
 	Delete(size);
 	clear = 0;
       }
@@ -830,16 +841,16 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
     } else if (SwigType_isfunction(element)) {
       DOH *parms, *p;
       int j, plen;
-      Append(result,"(");
+      StringAppend(result,"(");
       parms = SwigType_parmlist(element);
       plen = Len(parms);
       for (j = 0; j < plen; j++) {
 	p = SwigType_str(Getitem(parms,j),0);
-	Append(result,p);
+	StringAppend(result,p);
 	Delete(p);
-	if (j < (plen-1)) Append(result,",");
+	if (j < (plen-1)) StringAppend(result,",");
       }
-      Append(result,")");
+      StringAppend(result,")");
       Delete(parms);
     } else {
       String *bs = SwigType_namestr(element);
@@ -851,16 +862,16 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
   }
   Delete(elements);
   if (clear) {
-    cast = NewString("");
+    cast = NewStringEmpty();
   } else {
     cast = NewStringf("(%s)",result);
   }
   if (name) {
     if (isreference) {
       if (isarray) Clear(cast);
-      Append(cast,"*");
+      StringAppend(cast,"*");
     }
-    Append(cast,name);
+    StringAppend(cast,name);
   }
   Delete(result);
   Delete(tc);
@@ -877,21 +888,25 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
 String *SwigType_lcaststr(SwigType *s, const String_or_char *name) {
   String *result;
 
-  result = NewString("");
+  result = NewStringEmpty();
 
   if (SwigType_isarray(s)) {
-    Printf(result,"(%s)%s", SwigType_lstr(s,0),name);
+    String *lstr = SwigType_lstr(s,0);
+    Printf(result,"(%s)%s", lstr,name);
+    Delete(lstr);
   } else if (SwigType_isreference(s)) {
     String *str = SwigType_str(s,0);
     Printf(result,"(%s)", str);
     Delete(str);
     if (name) 
-      Printv(result,name,NIL);
+      StringAppend(result,name);
   } else if (SwigType_isqualifier(s)) {
-    Printf(result,"(%s)%s", SwigType_lstr(s,0),name);
+    String *lstr = SwigType_lstr(s,0);
+    Printf(result,"(%s)%s", lstr,name);
+    Delete(lstr);
   } else {
     if (name)
-      Append(result,name);
+      StringAppend(result,name);
   }
   return result;
 }
@@ -900,7 +915,8 @@ String *SwigType_lcaststr(SwigType *s, const String_or_char *name) {
 /* keep old mangling since Java codes need it */
 String *SwigType_manglestr_default(SwigType *s) {
   char *c;
-  String *result,*base;
+  String *result = 0;
+  String *base = 0;
   SwigType *lt;
   SwigType *sr = SwigType_typedef_qualified(s);
   SwigType *ss = SwigType_typedef_resolve_all(sr);
@@ -948,7 +964,7 @@ String *SwigType_manglestr_default(SwigType *s) {
     else if (!isalnum((int)*c)) *c = '_';
     c++;
   }
-  Append(result,base);
+  StringAppend(result,base);
   Insert(result,0,"_");
   Delete(lt);
   Delete(base);
@@ -969,26 +985,27 @@ String *SwigType_manglestr(SwigType *s) {
 void
 SwigType_typename_replace(SwigType *t, String *pat, String *rep) {
   String *nt;
-  int    i;
+  int    i, ilen;
   List   *elem;
 
   if (!Strstr(t,pat)) return;
 
-  if (Strcmp(t,pat) == 0) {
+  if (StringEqual(t,pat)) {
     Replace(t,pat,rep,DOH_REPLACE_ANY);
     return;
   }
-  nt = NewString("");
+  nt = NewStringEmpty();
   elem = SwigType_split(t);
-  for (i = 0; i < Len(elem); i++) {
+  ilen = Len(elem);
+  for (i = 0; i < ilen; i++) {
     String *e = Getitem(elem,i);
     if (SwigType_issimple(e)) {
-      if (Strcmp(e,pat) == 0) {
+      if (StringEqual(e,pat)) {
 	/* Replaces a type of the form 'pat' with 'rep<args>' */
 	Replace(e,pat,rep,DOH_REPLACE_ANY);
       } else if (SwigType_istemplate(e)) {
          /* Replaces a type of the form 'pat<args>' with 'rep' */
-	if (Strncmp(e,pat,Len(pat)) == 0) {
+	if (StringEqual(e,pat)) {
 	  String *repbase = SwigType_templateprefix(rep);
 	  Replace(e,pat,repbase,DOH_REPLACE_ID | DOH_REPLACE_FIRST);
 	  Delete(repbase);
@@ -996,19 +1013,20 @@ SwigType_typename_replace(SwigType *t, String *pat, String *rep) {
 	{
 	  String *tsuffix;
 	  List *tparms = SwigType_parmlist(e);
-	  int j;
+	  int j, jlen;
 	  String *nt = SwigType_templateprefix(e);
-	  Printv(nt,"<(",NIL);
-	  for (j = 0; j < Len(tparms); j++) {
+	  StringAppend(nt,"<(");
+	  jlen = Len(tparms);
+	  for (j = 0; j < jlen; j++) {
 	    SwigType_typename_replace(Getitem(tparms,j), pat, rep);
-	    Printv(nt,Getitem(tparms,j),NIL);
-	    if (j < (Len(tparms)-1)) Putc(',',nt);
+	    StringAppend(nt,Getitem(tparms,j));
+	    if (j < (jlen-1)) Putc(',',nt);
 	  }
 	  tsuffix = SwigType_templatesuffix(e);
 	  Printf(nt,")>%s", tsuffix);
 	  Delete(tsuffix);
 	  Clear(e);
-	  Append(e,nt);
+	  StringAppend(e,nt);
 	  Delete(nt);
 	  Delete(tparms);
 	}
@@ -1024,24 +1042,26 @@ SwigType_typename_replace(SwigType *t, String *pat, String *rep) {
 	Delete(rest);
       }
     } else if (SwigType_isfunction(e)) {
-      int j;
+      int j, jlen;
       List *fparms = SwigType_parmlist(e);
       Clear(e);
-      Printv(e,"f(",NIL);
-      for (j = 0; j < Len(fparms); j++) {
+      StringAppend(e,"f(");
+      jlen = Len(fparms);
+      for (j = 0; j < jlen; j++) {
 	SwigType_typename_replace(Getitem(fparms,j), pat, rep);
-	Printv(e,Getitem(fparms,j),NIL);
-	if (j < (Len(fparms)-1)) Putc(',',e);
+	StringAppend(e,Getitem(fparms,j));
+	if (j < (jlen-1)) Putc(',',e);
       }
-      Printv(e,").",NIL);
+      StringAppend(e,").");
       Delete(fparms);
     } else if (SwigType_isarray(e)) {
       Replace(e,pat,rep, DOH_REPLACE_ID);
     }
-    Append(nt,e);
+    StringAppend(nt,e);
   }
   Clear(t);
-  Append(t,nt);
+  StringAppend(t,nt);
+  Delete(nt);
   Delete(elem);
 }
 
@@ -1058,10 +1078,9 @@ SwigType_check_decl(SwigType *ty, const SwigType *decl) {
   t = SwigType_typedef_resolve_all(ty);
   t1 = SwigType_strip_qualifiers(t);
   t2 = SwigType_prefix(t1);
-  r = Cmp(t2,decl);
+  r = Equal(t2,decl);
   Delete(t);
   Delete(t1);
   Delete(t2);
-  if (r == 0) return 1;
-  return 0;
+  return r == 1;
 }
